<!DOCTYPE html>
<html>
    <head>
	<title>Example of Iconify usage in Django</title>
	<script type="text/javascript" src="{% url 'config.js' %}"></script>
	<script type="text/javascript" src="https://code.iconify.design/1/1.0.6/iconify.min.js"></script>
    </head>
    <body>
	<span class="iconify" data-icon="mdi:account" style="color:#00ff00"></span>
	<span class="iconify" data-icon="mdi:account-cash" style="color:#0000ff;font-size:48pt"></span>
	<span class="iconify" data-icon="mdi:seatbelt" style="color:#ff0000;font-size:72pt"></span>
    </body>
</head>
